using System;
using System.Collections.Generic;
using Produmex.Foundation.Collections;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.DataObjects;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Utilities;
using Produmex.Foundation.Data.DbClient;
using Produmex.Sbo.Logex.TrendTex.DataObjects;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// Reception
    /// </summary>
    public class WorkflowScript_ASNReceptionTrendTexScript : WorkflowInstanceScriptBase
    {
        /// <summary>
        /// Definitions
        /// </summary>
        private static class Definitions
        {
            /// <summary>
            /// The separator for separating strings values
            /// </summary>
            public const string SEPARATOR = ", ";

            /// <summary>
            /// Minimum number of characters when entering reason
            /// </summary>
            public const int MIMIMUM_NUMBER_OF_CHARACTERS_FOR_REASON = 10;

        }


        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;


        private CultureInfo current_DefaultCultureInfo
        {
            get { return DefaultCultureInfo.Get(); }
        }

        private PmxOseCompany currentPmxOseCompany
        {
            get { return PmxOseCompany.Get(); }
        }


        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        private WorkflowInstanceBase m_printItemLabelFlow;
        private bool m_printItemLabelFlow_BackRequested = false;

        private string m_printItemLabelFlow_LocationCode;
        private string m_printItemLabelFlow_NeededItemCode;
        private string m_printItemLabelFlow_NeededBatchNumber;
        private string m_printItemLabelFlow_NeededBatchNumber2;
        private DateTime? m_printItemLabelFlow_NeededBestBeforeDate;
        private int? m_printItemLabelFlow_NeededNrOfLabels;
        private double? m_printItemLabelFlow_EnteredQuantity;


        #region Constructors

        public WorkflowScript_ASNReceptionTrendTexScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {

        }

        #endregion

        #region WorkflowInstanceScriptBase Members

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            string initialErrorKey = null;
            string deviceID = session.DeviceInstanceId;
            SboUserInfo userInfo = sboProviderService.GetCurrentUser();
            Message msg = null;
            string query = null;
            bool goBack = false;

            bool askReprint = false;

            int numberOfReports = 0;
            int asnId = 0;

            string currentDockCode = null;
            string cardCode = null;

            //    LogisticUnitGoodsReceipt currentLogUnit = null;
            PmxOseWarehouseInfo whsInfo = null;
            PmxItemInfo itemInfo = null;

            PmxDictionary<string, PmxItemInfo> listOfItemInfos = null;

            DataSet dsDocks = null;
            DataSet dsASN = null;
            DataSet dsOpenASNLines = null;

            KeyValuePair<Collection<DocumentRef>, Collection<int>> listOfGeneratedDocEntry = new KeyValuePair<Collection<DocumentRef>, Collection<int>>();
            Collection<LogisticUnitGoodsReceiptTT> purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceiptTT>();
            List<int?> luids = new List<int?>();
            Collection<PmxDictionary<string, object>> listOfListOfParameters = new Collection<PmxDictionary<string, object>>();

            whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), deviceID);


            bool askUserToPrintLabels = false;

            
            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
             delegate(PmxDbConnection conn, object[] parameters)
             {
                
                 PmxPurchaseDeliveryNoteProvider prov = new PmxPurchaseDeliveryNoteProvider();
                 PmxDictionary<string, PmxParameterValue> pdParams = prov.GetParametersPurchaseOrderGenerator(conn);

                 askUserToPrintLabels = PmxParameterValueProvider.GetParameterValue<bool>(pdParams, PmxExtensionParameterCodes.PURCHASE_DELIVERY_ASK_TO_PRINT_LABEL, false);

                 return null;
             });




            //initialize sub flows
            m_printItemLabelFlow = Create("PrintItemLabelScript");
            m_printItemLabelFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<string>("LocationCode", this, "m_printItemLabelFlow_LocationCode"),
                new ReadOnlyBinder<string>("NeededItemCode", this, "m_printItemLabelFlow_NeededItemCode"),
                new ReadOnlyBinder<string>("NeededBatchNumber", this, "m_printItemLabelFlow_NeededBatchNumber"),
                new ReadOnlyBinder<string>("NeededBatchNumber2", this, "m_printItemLabelFlow_NeededBatchNumber2"),
                new ReadOnlyBinder<DateTime?>("NeededBestBeforeDate", this, "m_printItemLabelFlow_NeededBestBeforeDate"),
                new ReadOnlyBinder<int?>("NeededNrOfLabels", this, "m_printItemLabelFlow_NeededNrOfLabels"),
                new ReadWriteBinder<bool>("BackRequested", this, "m_printItemLabelFlow_BackRequested"),
                new ReadWriteBinder<double?>("EnteredQuantity", this, "m_printItemLabelFlow_EnteredQuantity")
                );


        Step_SelectDock:

            #region SELECT DOCK

            currentDockCode = null;

            // Get locations
            query = BuildQuery.GetCodeNameDockOrderByName(false, true, true, whsInfo.PmxWarehouseCode, deviceID);
            dsDocks = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsDocks) == 1 && !goBack)
            {
                currentDockCode = Convert.ToString(dsDocks.Tables[0].Rows[0][LocationDataSet.Columns.CODE.Name]);
                //reset warehouse info
                whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), currentDockCode);
                goto Step_OpenASNID;
            }
            goBack = false;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_LOCATION",
                "LocationDS", dsDocks
                ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;
            if (msg.Name.EndsWith(".LocationSelected"))
            {
                currentDockCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                //reset warehouse info
                whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), currentDockCode);

                goto Step_OpenASNID;
            }

            if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectDock;
            }

            //back requested
            return;

            #endregion

        Step_OpenASNID:

            #region SELECT OPEN ASN ID

            asnId = 0;
            cardCode = null;

            query = BuildQuery.GetOpenAdvanceShippingNoticesForReception(currentDockCode, whsInfo.PmxWarehouseCode, deviceID);
            dsASN = sboProviderService.RunView(false, null, null, query);

            //if (GetNumberOfRows(dsASN) > 0)
            //{

            //    goto Step_OpenASNID;
            //}

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.ISelectPOScreen),
              DefaultCultureInfo.Get(), BuildParamCollection(
              "InitialErrorKey", initialErrorKey,
              "TitleKey", "MSG_TITLE_SELECT_OPEN_ASN",
              "PODS", dsASN));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;

            if (msg.Name.EndsWith(".POSelected"))
            {
                asnId = ExtractParameter<int>(msg.Parameters, "docEntry");

                DataRow[] rows = dsASN.Tables[0].Select(PurchaseOrderDataSet.Columns.PO_NUMBER.Name + " = " + asnId.ToString());
                if (rows.Length > 0)
                {
                    cardCode = Convert.ToString(rows[0][PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name]);
                }
                goto Step_ShowItemsToReceive;
            }

            if (msg.Name.EndsWith("ISelectPOScreenReply.RefreshRequested"))
            {
                goto Step_OpenASNID;
            }

            goBack = true;
            goto Step_SelectDock;

            #endregion

        Step_ShowItemsToReceive:

            #region SHOW ITEMS TO RECEIVE

            query = BuildQuery.GetAllOpenASNLinesToReceive( asnId, cardCode, sboProviderService.GetDbTool() );
            dsOpenASNLines = sboProviderService.RunView(false, null, null, query);
            purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceiptTT>();

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IShowReceivingItemsScreen),
              DefaultCultureInfo.Get(), BuildParamCollection(
              "InitialErrorKey", initialErrorKey,
              "TitleKey", "MSG_TITLE_SHOW_ASN_LINES_TO_RECEIVE",
              "ProductDS", dsOpenASNLines));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;

            if (msg.Name.EndsWith(".ForwardRequested"))
            {

                goto Step_EnterDataIntoListToReceive;
            }
            goBack = true;
            goto Step_OpenASNID;

            #endregion

        Step_EnterDataIntoListToReceive:

            #region ENTER DATA INTO LIST TO RECEIVE

            //foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
            //{
            //    int? luid = Convertor.ConvertToInt32Nullable(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME]);
            //    if (!luids.Contains(luid))
            //    {
            //        luids.Add(luid);
            //    }
            //}

            //foreach (int? luid in luids)
            //{

            Collection<string> listOfItemCodes = new Collection<string>();

            foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
            {
                string tempItemCode = row[ProductDataSet.Columns.PRODUCT_CODE.Name].ToString();
                if (!listOfItemCodes.Contains(tempItemCode))
                {
                    listOfItemCodes.Add(tempItemCode);
                }
            }


            listOfItemInfos = sboProviderService.InvokeMethodWithDbConnection<PmxDictionary<string, PmxItemInfo>>(false, false, null, null, new DelegateWithPmxDbConnection<PmxDictionary<string, PmxItemInfo>>(ExecuteGetItemInfos), listOfItemCodes);


            // foreach (DataRow row in dsOpenASNLines.Tables[0].Select(string.Format("{0} = {1}", PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME,luid)))
            foreach (DataRow row in dsOpenASNLines.Tables[0].Rows)
            {
                //Use the first logistic unit. This because the code at this moment only supports
                //reception of one logistic unit that must be saved as a good receipt.
                LogisticUnitItemGoodReceiptTT logUnitItem = new LogisticUnitItemGoodReceiptTT();
                LogisticUnitGoodsReceiptTT logUnit = new LogisticUnitGoodsReceiptTT();

               

                logUnit.LogisticUnitId = Convertor.ConvertToInt32Nullable(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME]);

                logUnit.MasterLUID = Convertor.ConvertToInt32Nullable(row[PmxAdvanceShippingNoticeTable.Columns.MasterLogUnitIdentKey.NAME]);

                logUnit.SSCC = string.IsNullOrEmpty(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString()) ?
                                null : row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString();
                logUnit.SupplierPalletNumber = string.IsNullOrEmpty(row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString()) ?
                                null : row[PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME].ToString();

                logUnitItem.ItemCode = row[ProductDataSet.Columns.PRODUCT_CODE.Name].ToString();
                logUnitItem.AutoSelectPO = true;
                logUnitItem.UnitPrice = Convert.ToDouble(row[PmxAdvanceShippingNoticeTable.Columns.UnitPrice.NAME]);
                logUnitItem.QuantityPerUom = Convert.ToDouble(row[PmxAdvanceShippingNoticeTable.Columns.QuantityPerUom.NAME]);
                logUnitItem.NumBolt = Convert.ToInt32(row[PmxAdvanceShippingNoticeTable.Columns.QuantityPerUom.NAME]);
				double? boltLngt = Convertor.ConvertToDoubleNullable(row[PmxAdvanceShippingNoticeTable.Columns.UF1.NAME]);
				logUnitItem.BoltLngt = boltLngt.HasValue ? boltLngt.Value : 0;

                ////get the iteminfo
                if (listOfItemInfos.ContainsKey(logUnitItem.ItemCode))
                {
                    itemInfo = listOfItemInfos[logUnitItem.ItemCode];
                }
                else
                {
                    itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), logUnitItem.ItemCode);
                }
                    
                logUnit.AllowReceptionWithoutLUID = !itemInfo.CreateSsccOnReception;// && !logUnit.LogisticUnitId.HasValue;

               
                logUnitItem.Quantity = Convert.ToDouble(row[ProductDataSet.Columns.QUANTITY.Name]) * itemInfo.NumberOfItemsPerPurchaseUnit;

				logUnitItem.PurchaseDocRef = new DocumentRef(row[PmxPurchaseOrderLineTable.Columns.ObjectType.NAME].ToString(),
													Convert.ToInt32(row[PmxPurchaseOrderLineTable.Columns.DocEntry.NAME]));
                logUnitItem.PurchaseDocumentLineNum = Convertor.ConvertToInt32Nullable(row[ProductDataSet.Columns.LINE_NUMBER.Name]);

                logUnitItem.BatchNumber1 = row[ProductDataSet.Columns.BATCH1.Name].ToString();
                logUnitItem.BatchNumber2 = row[ProductDataSet.Columns.BATCH2.Name].ToString();
                if (!string.IsNullOrEmpty(row[ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString()))
                {
                    logUnitItem.BestBeforeDate = DateTime.ParseExact(row[ProductDataSet.Columns.BEST_BEFORE_DATE.Name].ToString(), "yyyyMMdd", null);
                }

				if (itemInfo.HasSAPSerialNumber ||
						itemInfo.HasSerialNumber)
				{
					sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
					delegate (PmxDbConnection conn, object[] parameters)
					{
						PmxSerialNumberLinkToDocumentProvider seldProv = new PmxSerialNumberLinkToDocumentProvider(conn);
						//Get the serial numbers
						PmxDictionary<string, int> serialNumberList = seldProv.GetSerialNumbersLinkedToDocument(Data.BusinessObjects.Definitions.PmxObjectTypes.PmxAdvanceShippingNotice, Convert.ToInt32(row[ProductDataSet.Columns.INTERNAL_KEY.Name]), null);
						if (serialNumberList != null &&
							serialNumberList.Keys.Count > 0)
						{
							logUnitItem.ListOfSerialNumbers = new Collection<string>(serialNumberList.Keys.ToList());
						}

						return null;
					}));


				}
                
                //Instead of adding the logUnitItem we set it like this 
                //so go back in select location is possible
                //If we want multiple LogUnitItems then go back on Select Location must not be possible
                if (logUnit.ItemsOnLogisticUnit.Count.Equals(0))
                {
                    logUnit.ItemsOnLogisticUnit.Add(logUnitItem);
                }
                else
                {
                    logUnit.ItemsOnLogisticUnit[0] = logUnitItem;
                }

                purchaseDeliveryNoteList.Add(logUnit);
            }
        // }

            #endregion

        Step_ReceiveItems:

            #region RECEIVE ITEMS

            // Inform the user items are receiving
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "MessageKey", "MSG_RECEIVING_ALL_ITEMS",
                    "NoTranslationOfMessageKey", false,
                    "ShowButton", false));


            try
            {
                listOfGeneratedDocEntry = sboProviderService.InvokeMethodWithDbConnection<KeyValuePair<Collection<DocumentRef>, Collection<int>>>(true, false, null, null, new DelegateWithPmxDbConnection<KeyValuePair<Collection<DocumentRef>, Collection<int>>>(
                    delegate(PmxDbConnection conn, object[] parameters)
                    {
                        Produmex.Sbo.Logex.TrendTex.Providers.PmxAdvanceShippingNoticeProvider prov = new Produmex.Sbo.Logex.TrendTex.Providers.PmxAdvanceShippingNoticeProvider((SboConnection)conn);
                        return prov.AddPurchaseDeliveryNoteListOfLogisticUnitsForASNReception(cardCode, currentDockCode, asnId, purchaseDeliveryNoteList);
                    }));


               // purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceipt>();
                //listOfReasonCodeByItem = new PmxDictionary<string, ReasonInfo>();

                ////show items have been booked
                //session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                //    DefaultCultureInfo.Get(), BuildParamCollection(
                //     "MessageKey", "MSG_RECEPTION_HAS_BEEN_BOOKED",
                //        "ShowButton", true));
                //WaitForMessage();

                goto Step_PrintItemLabel;
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (ex is ProdumexException)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //ask user if he wants to try again after error
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_ReceiveItems;
                }
                purchaseDeliveryNoteList = new Collection<LogisticUnitGoodsReceiptTT>();
                goto Step_OpenASNID;
            }

        //goto Step_PrintDeliveryDocument;

        #endregion

        Step_PrintItemLabel:

            #region PRINT ITEM LABEL

            bool alreadyAskedToPrint = false;

            foreach (LogisticUnitGoodsReceiptTT logUnit in purchaseDeliveryNoteList)
            {
                foreach (LogisticUnitItemGoodReceiptTT logItemUnit in logUnit.ItemsOnLogisticUnit)
                {
                    itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxItemInfo>(ExecuteGetItemInfo), logItemUnit.ItemCode);

                    int? numberOfItemLabels = 0;

                    if (itemInfo.PrintItemLabel)
                    {
                        if (currentPmxOseCompany.GoodsReceiptAutoPrintItemLabels)
                        {
                            if (askUserToPrintLabels &&
                                !alreadyAskedToPrint)
                            {
                                //ask user if he wants to print item labels
                                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                    DefaultCultureInfo.Get(), BuildParamCollection(
                                        "TitleKey", "MSG_TITLE_PRINT_ITEM_LABEL",
                                        "MessageKey", "MSG_QUESTION_PRINT_ITEM_LABEL"));
                                msg = WaitForMessage();

                                if (msg.Name.EndsWith(".Yes"))
                                {
                                    numberOfItemLabels = Convert.ToInt32(Math.Ceiling(logItemUnit.Quantity));
                                }
                                else
                                {
                                    goto Step_PrintGoodsReceiptPONewSSCC;
                                }
                            }

                            numberOfItemLabels = Convert.ToInt32(Math.Ceiling(logItemUnit.Quantity));
                        }
                        else
                        {
                            if (!alreadyAskedToPrint)
                            {
                                //ask user if he wants to print item labels
                                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                    DefaultCultureInfo.Get(), BuildParamCollection(
                                        "TitleKey", "MSG_TITLE_PRINT_ITEM_LABEL",
                                        "MessageKey", "MSG_QUESTION_PRINT_ITEM_LABEL"));
                                msg = WaitForMessage();

                                if (msg.Name.EndsWith(".Yes"))
                                {
                                    //numberOfItemLabels = Convert.ToInt32(Math.Ceiling(quantity));
                                    numberOfItemLabels = null;
                                }
                                else
                                {
                                    goto Step_PrintGoodsReceiptPONewSSCC;
                                }
                            }
                            else
                            {
                                numberOfItemLabels = null;
                            }

                        }

                        // print the labels
                        //Set the data into binders
                        this.m_printItemLabelFlow_LocationCode = currentDockCode;
                        this.m_printItemLabelFlow_NeededBatchNumber = logItemUnit.BatchNumber1;
                        this.m_printItemLabelFlow_NeededBatchNumber2 = logItemUnit.BatchNumber2;
                        this.m_printItemLabelFlow_NeededBestBeforeDate = logItemUnit.BestBeforeDate;
                        this.m_printItemLabelFlow_NeededItemCode = itemInfo.ItemCode;
                        this.m_printItemLabelFlow_NeededNrOfLabels = numberOfItemLabels;
                        this.m_printItemLabelFlow_EnteredQuantity = logItemUnit.Quantity;

                        //call flow
                        base.StartSubFlowAndWait(m_printItemLabelFlow, true);
                        goBack = false;
                        if (m_printItemLabelFlow_BackRequested)
                        {
                            goto Step_OpenASNID;
                        }
                    }


                }
            }
            goto Step_PrintGoodsReceiptPONewSSCC;

            #endregion

        Step_PrintGoodsReceiptPONewSSCC:

            #region PRINT GOODS RECEIPT PO NEW SSCC

            foreach (LogisticUnitGoodsReceiptTT logUnit in purchaseDeliveryNoteList)
            {
               // foreach (LogisticUnitItemGoodReceipt logItemUnit in logUnit.ItemsOnLogisticUnit)
                {
                    if (!logUnit.IsMoveToLockedStockLocation &&
                        logUnit.LogisticUnitId.HasValue)
                    {

                        try
                        {
                            PmxDictionary<string, object> listOfParameters = new PmxDictionary<string, object>();

                            string tempObjType = SboObjectTypes.PurchaseOrder;
                            int tempDocEntry = -1;                       
                            
                            listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.LUID.NAME, logUnit.LogisticUnitId);

                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.CardCode.NAME, cardCode);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.PurchaseDocObjType.NAME, tempObjType);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.PurchaseDocEntry.NAME, tempDocEntry);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.SSCC.NAME, logUnit.SSCC);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.InternalLUID.NAME, logUnit.LogisticUnitId);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.LocationCode.NAME, string.IsNullOrEmpty(logUnit.SpecificLocationCode) ? currentDockCode : logUnit.SpecificLocationCode);


                            //string tempItemCode = null;
                            //string tempBatch = null;
                            //string tempBatch2 = null;
                            //DateTime? tempBestBeforeDate = null;
                            //double? tempQuantity = 0;
                            //Collection<PackagingTypeInfo> tempListOfPackagingTypes = null;
                            //bool isFirst = true;

                            //foreach (LogisticUnitItemGoodReceipt logItemUnit in logUnit.ItemsOnLogisticUnit)
                            //{

                            //    tempObjType = logItemUnit.PurchaseDocRef.DocType;
                            //    tempDocEntry = logItemUnit.PurchaseDocRef.DocEntry;   

                            //    if (isFirst)
                            //    {
                            //        if (!logItemUnit.IsLogisticCarrierForLogisticUnit || logItemUnit.Quantity > 1)
                            //        {
                            //            //we need to fill the current data
                            //            tempItemCode = logItemUnit.ItemCode;
                            //            tempBatch = logItemUnit.BatchNumber1;
                            //            tempBatch2 = logItemUnit.BatchNumber2;
                            //            tempBestBeforeDate = logItemUnit.BestBeforeDate;
                            //            tempQuantity = logItemUnit.Quantity;
                            //            tempListOfPackagingTypes = logItemUnit.FullListOfPackagingTypes;

                            //            isFirst = false;
                            //        }
                            //    }
                            //    else if (tempItemCode != logItemUnit.ItemCode && (!logItemUnit.IsLogisticCarrierForLogisticUnit || logItemUnit.Quantity > 1))
                            //    {
                            //        tempItemCode = null;
                            //        tempBatch = null;
                            //        tempBatch2 = null;
                            //        tempBestBeforeDate = null;
                            //        tempQuantity = null;
                            //        tempListOfPackagingTypes = null;
                            //        break;
                            //    }
                            //    else if (!logItemUnit.IsLogisticCarrierForLogisticUnit)
                            //    {
                            //        //check if bestbefore date and batchnum correspond
                            //        if (tempBatch != logItemUnit.BatchNumber1 ||
                            //            tempBatch2 != logItemUnit.BatchNumber2)
                            //        {
                            //            tempBatch = null;
                            //            tempBatch2 = null;
                            //        }

                            //        if (tempBestBeforeDate != logItemUnit.BestBeforeDate)
                            //        {
                            //            tempBestBeforeDate = null;
                            //        }

                            //        tempQuantity += logItemUnit.Quantity;
                            //    }
                            //}


                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.Quantity.NAME, tempQuantity);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.BatchNumber.NAME, tempBatch);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.BestBeforeDate.NAME, tempBestBeforeDate);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.ItemCode.NAME, tempItemCode);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.BatchNumber2.NAME, tempBatch2);

                            //string tempPackagingTypeText = null;
                            //string tempQuantityByPackText = null;
                            //if (tempListOfPackagingTypes != null &&
                            //    tempQuantity.HasValue)
                            //{
                            //    tempPackagingTypeText = this.GetTransactionalPackagingTypesText(tempQuantity.Value, null, tempListOfPackagingTypes);
                            //    tempQuantityByPackText = this.GetQuantityInPackagingTypesText(tempListOfPackagingTypes);
                            //}

                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.PackagingTypeText.NAME, tempPackagingTypeText);
                            //listOfParameters.Add(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeLogisticUnitDefinition.Parameters.QuantityByPackagingTypeText.NAME, tempQuantityByPackText);


                            listOfListOfParameters.Add(listOfParameters);

                        }
                        catch (Produmex.Foundation.ProdumexException ex)
                        {
                            s_log.Error(ex.ToString());
                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection(
                                    "InitialErrorKey", initialErrorKey,
                                    "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                    "MessageAddendum", ex.Message,
                                    "PmxError", ex.Error,
                                    "ShowButton", true));

                            msg = WaitForMessage();
                        }
                        catch (Exception ex)
                        {
                            s_log.Error(ex.ToString());
                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection(
                                    "InitialErrorKey", initialErrorKey,
                                    "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                    "MessageAddendum", ex.Message,
                                    "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                    "ShowButton", true));

                            msg = WaitForMessage();
                        }

                    }
                }
            }


            //only print if not moving to locked stock location
            //When no parameters are filled in, stock will be moved to locked stock location
            if (listOfListOfParameters.Count > 0)
            {
                //print the label
                bool proceed = false;
                while (!proceed)
                {
                    numberOfReports = 0;

                    try
                    {

                        if (askUserToPrintLabels)
                        {
                            //ask user if he wants to print logistic label
                            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                DefaultCultureInfo.Get(), BuildParamCollection(
                                    "TitleKey", "MSG_TITLE_PRINT_LABEL",
                                    "MessageKey", "MSG_QUESTION_PRINT_LABEL"));
                            msg = WaitForMessage();

                            if (!msg.Name.EndsWith(".Yes"))
                            {
                                break;
                            }

                        }

                        //trigger event
                        numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
                         delegate(PmxDbConnection conn, object[] parameters2)
                         {
                             PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                             return printProv.PrintReportsForEventWithParametersCollection(PmxPrintReportEventType.ReceptionNewLogisticUnitIdentified,
                             currentDockCode, deviceID, listOfListOfParameters, false, out askReprint);
                         });

                    }
                    catch (Produmex.Foundation.ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", ex.Error,
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }

                    if (numberOfReports > 0 && askReprint)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                                "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".No"))
                        {
                            proceed = true;
                        }

                    }
                    else
                    {
                        proceed = true;
                    }

                }
            }


            goto Step_PrintDeliveryDocument;

            #endregion

        Step_PrintDeliveryDocument:

            #region PRINT DELIVERY DOCUMENT

            numberOfReports = 0;
            try
            {
                foreach (DocumentRef docRef in listOfGeneratedDocEntry.Key)
                {
                    //trigger event
                    numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null,
                         delegate(PmxDbConnection conn, object[] parameters2)
                         {
                             PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                             return printProv.PrintReportsForEvent(PmxPrintReportEventType.ReceptionPurchaseDeliveryNoteCreated,
                             currentDockCode, deviceID, docRef.DocEntry, out askReprint);
                         });
                }

            }
            catch (Produmex.Foundation.ProdumexException ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", ex.Error,
                        "ShowButton", true));

                msg = WaitForMessage();
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                        "MessageAddendum", ex.Message,
                        "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                        "ShowButton", true));

                msg = WaitForMessage();
            }

            goto Step_DocumentsPrinted;

            #endregion


        Step_DocumentsPrinted:

            #region DOCUMENTS PRINTED

            if (numberOfReports > 0 && askReprint)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT",
                        "TitleKey", "MSG_TITLE_REPRINT_DOCUMENT"));

                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_PrintDeliveryDocument;
                }
            }
            goto Step_OpenASNID;

            #endregion

            //    return;
        }

        #endregion

        #region HELPERS



        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }



        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the get item infos.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxDictionary<string, PmxItemInfo> ExecuteGetItemInfos(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfoInDictionary((Collection<string>)parameters[0]);
        }



        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the need to enter supplier pallet number.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static bool ExecuteNeedToEnterSupplierPalletNumber(PmxDbConnection conn, object[] parameters)
        {
            PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
            return prov.NeedToEnterSupplierPalletNumber((string)parameters[0]);
        }

        /// <summary>
        /// Executes the check allow multiple identical log units.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxBusinessPartnerInfo ExecuteGetBusinessPartnerInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxBusinessPartnerProvider prov = new PmxBusinessPartnerProvider((SboConnection)conn);
            return prov.GetBusinessPartnerInfo((string)parameters[0]);
        }

        private string GetTransactionalPackagingTypesText(double quantity, string uom, Collection<PackagingTypeInfo> listOfTransactionalPackagingTypes)
        {

            if (listOfTransactionalPackagingTypes == null ||
                listOfTransactionalPackagingTypes.Count == 0)
            {
                return string.Format("# {0} {1}", quantity.ToString(), uom);
            }

            //we found packagingTypes
            double currentQty = quantity;
            StringBuilder sb = new StringBuilder();
            string separator = "";
            //loop from end
            for (int iLoop = 0; iLoop < listOfTransactionalPackagingTypes.Count; iLoop++)
            {
                double qty = currentQty / listOfTransactionalPackagingTypes[iLoop].QuantityPerPack;

                qty = Math.Floor(qty);


                sb.AppendFormat("{0}{1} {2}",
                    separator, qty.ToString(), listOfTransactionalPackagingTypes[iLoop].PackagingTypeName);

                currentQty -= qty * listOfTransactionalPackagingTypes[iLoop].QuantityPerPack;

                separator = Definitions.SEPARATOR;

            }

            return sb.ToString();


        }


        private string GetQuantityInPackagingTypesText(Collection<PackagingTypeInfo> listOfTransactionalPackagingTypes)
        {

            if (listOfTransactionalPackagingTypes == null ||
                listOfTransactionalPackagingTypes.Count == 0)
            {
                return null;
            }

            //we found packagingTypes
            StringBuilder sb = new StringBuilder();
            string separator = "";



            for (int iLoop = 0; iLoop < listOfTransactionalPackagingTypes.Count - 1; iLoop++)
            {

                sb.AppendFormat("{0}1 {1} = {2} {3}", separator, listOfTransactionalPackagingTypes[iLoop].PackagingTypeName, listOfTransactionalPackagingTypes[iLoop].QuantityPerPack / listOfTransactionalPackagingTypes[iLoop + 1].QuantityPerPack, listOfTransactionalPackagingTypes[iLoop + 1].PackagingTypeName);


                separator = Definitions.SEPARATOR;

            }

            return sb.ToString();

        }

        #endregion

        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {


            private static SboBooleanConvertor m_sboBooleanConvertor = null;
            private static SboBooleanConvertor BooleanConvertor
            {
                get
                {
                    if (m_sboBooleanConvertor == null)
                    {
                        m_sboBooleanConvertor = new SboBooleanConvertor();
                    }
                    return m_sboBooleanConvertor;
                }
            }

            /// <summary>
            /// Gets the name and the code of the  location, ordered by name.
            /// </summary>
            /// <param name="isLoading">Is the dock for loading? When null, no check will be done on this value</param>
            /// <param name="isUnloading">Is the dock for unloading? When null, no check will be done on this value</param>
            /// <param name="useAnd">if set to <c>true</c> the operand between Loading and Unloading is 'AND'.
            /// Otherwise it is 'OR'.</param>
            /// <param name="pmxWarehouseCode">The PMX warehouse code.</param>
            /// <param name="deviceID">The device ID.</param>
            /// <returns>The sql statement to get the dock</returns>
            public static string GetCodeNameDockOrderByName(bool? isLoading, bool? isUnloading, bool useAnd, string pmxWarehouseCode, string deviceID)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);


                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));

                //inner join PmxOseDock - PmxOse
                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

                //inner join PmxOseDock - PmxOseStorageLocation
                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.IsManagedByProdumex.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                strBuilder.Append("\r\n WHERE ");

                //Dock must be active
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                if (!String.IsNullOrEmpty(pmxWarehouseCode) || !String.IsNullOrEmpty(deviceID))
                {
                    strBuilder.Append(" AND ( ");
                    if (!String.IsNullOrEmpty(pmxWarehouseCode))
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pmxWarehouseCode, CultureInfo.InvariantCulture));
                    }
                    if (!String.IsNullOrEmpty(deviceID))
                    {
                        if (!String.IsNullOrEmpty(pmxWarehouseCode)) strBuilder.Append(" OR ");

                        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                            SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                    }
                    strBuilder.Append(" )");
                }

                if (isUnloading.HasValue || isLoading.HasValue)
                {
                    strBuilder.Append(" AND (");

                    if (isUnloading.HasValue)
                    {
                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.IsUnloadingDock.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(BooleanConvertor.ConvertBOPropertyToColumn(isUnloading.Value), CultureInfo.InvariantCulture));
                    }

                    if (isLoading.HasValue)
                    {
                        if (isUnloading.HasValue)
                        {
                            strBuilder.Append(useAnd ? " AND " : " OR ");
                        }

                        SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.IsLoadingDock.NAME);
                        strBuilder.Append(" = ");
                        strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(BooleanConvertor.ConvertBOPropertyToColumn(isLoading.Value), CultureInfo.InvariantCulture));
                    }

                    strBuilder.Append(" )");
                }

                strBuilder.Append("\r\n ORDER BY ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME);

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the open advance shipping notices for reception.
            /// </summary>
            /// <param name="dockCode">The dock code.</param>
            /// <param name="pmxWarehouseCode">The PMX warehouse code.</param>
            /// <param name="deviceID">The device unique identifier.</param>
            /// <returns></returns>
            internal static string GetOpenAdvanceShippingNoticesForReception(string dockCode, string pmxWarehouseCode, string deviceID)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.AppendFormat("SELECT DISTINCT {0} AS {1}",
                    SqlCommandHelper.FormatAndEscapeColumnValue(PmxAdvanceShippingNoticeTable.NAME, CultureInfo.InvariantCulture),
                     SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name)
                     );
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME, PurchaseOrderDataSet.Columns.PO_NUMBER.Name));
                strBuilder.AppendFormat(", CAST( {0} AS NVARCHAR ) AS {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
                    SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.PO_NUMBER_ASSTRING.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME, PurchaseOrderDataSet.Columns.PO_ID.Name));
                
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.CardCode.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name));
                strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.CardCode.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_NAME.Name));
                strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.CardName.NAME, PurchaseOrderDataSet.Columns.SUPPLIER_REFERENCE.Name));
                


                strBuilder.AppendFormat(", CAST( {0} AS NVARCHAR ) AS {1}",
                    SqlCommandHelper.FormatAndEscapeColumnValue(null, CultureInfo.InvariantCulture),
                    SqlCommandHelper.EscapeColumnValue(PurchaseOrderDataSet.Columns.SUPPLIER_LINE_REF.Name));

                strBuilder.AppendFormat("\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxAdvanceShippingNoticeTable.NAME));


				strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseLine.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME)
					);

				strBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPurchaseOrderTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.ObjType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME));

				strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
				  SqlCommandHelper.EscapeTableName(PmxPurchaseInvoiceLineTable.NAME),
				  SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
				  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseEntry.NAME),
				  SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.LineNum.NAME),
				  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseLine.NAME),
				  SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ObjectType.NAME),
				  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME)
				  );

				strBuilder.AppendFormat("\r\n LEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
				strBuilder.AppendFormat(" ON {0} = {1}",
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
					SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.ObjType.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME));




                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                  SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME),
					SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.ItemCode.NAME));



                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LineStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

				strBuilder.Append("\r\n  AND (");
				{
					{
						strBuilder.Append("\r\n  (");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.DocStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Canceled.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Confirmed.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));
						strBuilder.Append("\r\n  )");
					}
					strBuilder.Append("\r\n  OR ");
					{
						strBuilder.Append("\r\n  (");

						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceTable.Columns.Canceled.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceTable.Columns.Confirmed.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.LineStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));


						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.InventoryStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));


						strBuilder.Append("\r\n  )");
					}
				}
				strBuilder.Append(")");

                strBuilder.Append("\r\n AND ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				strBuilder.Append("\r\n AND ");
				strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME));
				strBuilder.Append(" IS NOT NULL ");


				strBuilder.AppendFormat("\r\n GROUP BY {0}, {1}, {2}, {3}",
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
					 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.CardCode.NAME),
					 SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.CardName.NAME),
                     SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME)
                     );


                //strBuilder.AppendFormat("\r\n ORDER BY {0}",
                //    SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.DOC_OBJ_TYPE.Name));
                //strBuilder.AppendFormat(", {0}",
                //    SqlCommandHelper.EscapeColumnName(PurchaseOrderDataSet.Columns.SUPPLIER_CODE.Name));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all open asn lines automatic receive.
            /// </summary>
            /// <param name="asnId">The asn unique identifier.</param>
            /// <param name="cardCode">The card code.</param>
            /// <param name="dbTool">The db tool.</param>
            /// <returns></returns>
            internal static string GetAllOpenASNLinesToReceive(int asnId, string cardCode, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.AppendFormat("SELECT {0}, {1}, {2}, {3}, {4}, {5}, {6}, {7}, {8}, {9}, {10}, {11}, {12}, {13}, {14}, {15}, {16}, {17}, {18}, {19}",
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
                        SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceLineTable.NAME, SboPurchaseInvoiceLineTable.Columns.ObjectType.NAME),
                        SqlCommandHelper.EscapeColumnName(PmxPurchaseOrderTable.Columns.ObjType.NAME)
                         ),
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocNum.NAME),
                        SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocNum.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.DOC_NUM.Name)
                         ),
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.LineNum.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.LINE_NUMBER.Name)
                         ),
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ItemCode.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_CODE.Name)
                         ),
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.Description.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.Description.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name)
                         ),

                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.Quantity.NAME, ProductDataSet.Columns.QUANTITY.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTable.NAME, PmxItemTable.Columns.InventoryUom.NAME, ProductDataSet.Columns.UOM.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, ProductDataSet.Columns.BATCH1.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME, ProductDataSet.Columns.BATCH2.Name),

                    dbTool.DbServerType == DbServerType.HANA ?
                    string.Format("TO_VARCHAR ({0}, 'YYYYMMDD') AS {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.BEST_BEFORE_DATE.Name)
                        ) :
                    string.Format("CONVERT(NVARCHAR(255), {0}, 112) AS {1}",
                        SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.BEST_BEFORE_DATE.Name)
                        ),
                    string.Format("" + strBuilder.IFNULL + "( " + strBuilder.IFNULL + "({0},{1}), {2} ) AS {3}",
                        SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SupplierLogisticUnitID.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                        SqlCommandHelper.FormatAndEscapeColumnValue(string.Empty, CultureInfo.InvariantCulture),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.USER_INFO.Name)
                        ),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SupplierLogisticUnitID.NAME),
                    string.Format("COALESCE ({0}, {1} ) AS {2}",
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
                        SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
                        SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.DOC_ENTRY.Name)
                         ),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.InternalKey.NAME, ProductDataSet.Columns.INTERNAL_KEY.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.MasterLogUnitIdentKey.NAME, PmxAdvanceShippingNoticeTable.Columns.MasterLogUnitIdentKey.NAME),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UnitPrice.NAME, PmxAdvanceShippingNoticeTable.Columns.UnitPrice.NAME),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.UF1.NAME, PmxAdvanceShippingNoticeTable.Columns.UF1.NAME),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.QuantityPerUom.NAME, PmxAdvanceShippingNoticeTable.Columns.QuantityPerUom.NAME)

                    ); 

                strBuilder.AppendFormat("\r\n FROM {0}",
                   SqlCommandHelper.EscapeTableName(PmxAdvanceShippingNoticeTable.NAME));

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
                    SqlCommandHelper.EscapeTableName(PmxPurchaseOrderLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseLine.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.ObjectType.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME)
                    );

                strBuilder.AppendFormat("\r\n LEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxPurchaseOrderTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderLineTable.NAME, PmxPurchaseOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseOrderTable.NAME, PmxPurchaseOrderTable.Columns.ObjType.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME));

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
              SqlCommandHelper.EscapeTableName(PmxPurchaseInvoiceLineTable.NAME),
              SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
              SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseEntry.NAME),
              SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.LineNum.NAME),
              SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseLine.NAME),
              SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.ObjectType.NAME),
              SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME)
              );

                strBuilder.AppendFormat("\r\n LEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPurchaseInvoiceLineTable.NAME, PmxPurchaseInvoiceLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboPurchaseInvoiceTable.NAME, SboPurchaseInvoiceTable.Columns.ObjType.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.BaseType.NAME));


                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} ",
                  SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.ItemTransactionalInfoKey.NAME)
                  );

                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} ",
                SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.LogUnitIdentKey.NAME)
                );


                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
              SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                       SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.GroupBy.NAME),
                   SqlCommandHelper.FormatAndEscapeColumnValue(asnId, CultureInfo.InvariantCulture));


                if (!string.IsNullOrEmpty(cardCode))
                { 
                strBuilder.AppendFormat("\r\n AND {0} = {1}",
                     SqlCommandHelper.EscapeTableColumnName(PmxAdvanceShippingNoticeTable.NAME, PmxAdvanceShippingNoticeTable.Columns.CardCode.NAME),
                  SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, CultureInfo.InvariantCulture));
                }

				strBuilder.Append("\r\n  AND (");
				{
					{
						strBuilder.Append("\r\n  (");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.DocStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Canceled.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderTable.Columns.Confirmed.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseOrderLineTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseOrderLineTable.Columns.LineStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));
						strBuilder.Append("\r\n  )");
					}
					strBuilder.Append("\r\n  OR ");
					{
						strBuilder.Append("\r\n  (");
						
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceTable.Columns.Canceled.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceTable.Columns.Confirmed.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.LineStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));


						strBuilder.Append("\r\n  AND ");
						strBuilder.Append(SqlCommandHelper.EscapeTableName(SboPurchaseInvoiceLineTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboPurchaseInvoiceLineTable.Columns.InventoryStatus.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));


						strBuilder.Append("\r\n  )");
					}
				}
				strBuilder.Append(")");

				strBuilder.Append("\r\n AND ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.InventoryItem.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

				return strBuilder.ToString();
			}
        }


        #endregion

    }


}
